#include <iostream>
#include "Slider.h"

#define SLIDER_WIDTH (14)
#define SLIDER_HEIGHT (10)

#define SLIDER_LEFT (2 * (SLIDER_WIDTH/2))
#define SLIDER_RIGHT (DISPLAY_COLS - SLIDER_LEFT)
#define SLIDER_LINE_WIDTH_MAX (SLIDER_RIGHT - SLIDER_LEFT)
#define SLIDER_LINE_HEIGHT (4)
#define SLIDER_Y (DISPLAY_ROWS - 30)

Slider::Slider (double low, double high, double start, double increment,
                const char *low_tag, const char *high_tag,
                Window *parent, bool show_center, const char *name)
    : Window (parent, name, 3), _low (low), _high (high), _cur (start),
      _increment (increment), _range (high - low),
      _positions (((int)((high - low)/increment))), _show_center (show_center),
      _high_tag (high_tag), _low_tag (low_tag), enabled (true)
{
    font_t font = font_helvetica_10;
    drawTitledWindow (font, name);
    draw_labels ();
    draw ();
    refresh ();
}

void
Slider::draw_labels (void)
{
    font_t font = font_helvetica_10;
    if (_positions > (SLIDER_LINE_WIDTH_MAX/2)) {
        throw "Sorry, too many positions required.";
    }
    slider_width = (SLIDER_LINE_WIDTH_MAX / _positions) * _positions;
    slider_x = (DISPLAY_COLS - slider_width) / 2;

    // Handle the labels
    int xpos = slider_x - fontStrLen (font, _low_tag)/2;
    if (xpos < 2) xpos = 2;
    writeString (xpos, SLIDER_Y + FONT_HEIGHT (font) + 2, font, _low_tag);

    xpos = slider_x + slider_width - fontStrLen (font, _high_tag)/2;
    if (xpos + fontStrLen (font, _high_tag) > DISPLAY_COLS - 2)
        xpos = DISPLAY_COLS - 2 - fontStrLen (font, _high_tag);

    writeString (xpos, SLIDER_Y + FONT_HEIGHT (font) + 2, font, _high_tag);
}

void
Slider::draw (void)
{
    int space = slider_width / _positions;
    int pos = ((int) ((_cur - _low) / _increment));
    font_t font = font_helvetica_10;

    // Clear the slider area
    fillRect (1, SLIDER_Y - (SLIDER_HEIGHT/2) - 5 - FONT_HEIGHT (font),
              DISPLAY_COLS - 2, SLIDER_HEIGHT + 10 + FONT_HEIGHT (font), true);

    // Draw the slider line
    drawRect (slider_x, SLIDER_Y, slider_width + 1, SLIDER_LINE_HEIGHT, 1);

    // Draw the increment lines
    for (int i = 0; i <= _positions; i++) {
        drawRect (slider_x + (i * space), SLIDER_Y, 1, SLIDER_LINE_HEIGHT, 1);
    }

    // Draw the center line
    if (_show_center) {
        int center = (int)((_range/2)/_increment);
        drawRect (slider_x + (center *space),
                        SLIDER_Y + SLIDER_LINE_HEIGHT/2 - SLIDER_HEIGHT/2 - 2,
                        1, SLIDER_HEIGHT + 4, 1);
    }

    // Draw the slider for the current position
    int slider_center = slider_x + (pos * space);
    // Clears the interior of the box
    fillRect (slider_center - (SLIDER_WIDTH/2),
              SLIDER_Y + (SLIDER_LINE_HEIGHT/2) - (SLIDER_HEIGHT/2),
              SLIDER_WIDTH, SLIDER_HEIGHT, true);

    // Outer box
    drawRoundedRect (slider_center - (SLIDER_WIDTH/2),
              SLIDER_Y + (SLIDER_LINE_HEIGHT/2) - (SLIDER_HEIGHT/2),
              SLIDER_WIDTH, SLIDER_HEIGHT, 1);

    // Inner box
    drawRoundedRect (slider_center - (SLIDER_WIDTH/2) + 2,
              SLIDER_Y + (SLIDER_LINE_HEIGHT/2) - (SLIDER_HEIGHT/2),
              SLIDER_WIDTH - 4, SLIDER_HEIGHT, 1);

    char tmp[5];
    snprintf (tmp, sizeof (tmp), "%d", (int)_cur);
    int len = fontStrLen (font, tmp);
    writeString (slider_center - len/2,
        SLIDER_Y - (SLIDER_HEIGHT - SLIDER_LINE_HEIGHT)/2 -  FONT_HEIGHT (font),
        font, tmp);
}

void
Slider::draw_disabled (void)
{
    int space = slider_width / _positions;
    int pos = ((int) ((_cur - _low) / _increment));
    font_t font = font_helvetica_10;

    int xpos = slider_x - fontStrLen (font, _low_tag)/2;
    // Clear the slider area
    fillRect (1, SLIDER_Y - (SLIDER_HEIGHT/2) - 5 - FONT_HEIGHT (font),
              DISPLAY_COLS - 3, SLIDER_HEIGHT + 10 + FONT_HEIGHT (font), true);

    fillRect (xpos, SLIDER_Y + FONT_HEIGHT (font) + 2, DISPLAY_COLS - xpos - 1, FONT_HEIGHT (font), true);
    writeString (10, SLIDER_Y - (SLIDER_HEIGHT - SLIDER_LINE_HEIGHT)/2 -  FONT_HEIGHT (font), font, "Feature Unavailable");
}

bool
Slider::_onKnobButton (bool pushedAgain)
{
    back ();
}

bool
Slider::_onKnobLeft (bool pushedAgain)
{
    resetTimer ();
    if (enabled && _cur - _increment >= _low) {
        _cur -= _increment;
        update (_cur);
        draw ();
        refresh ();
    }
}

bool
Slider::_onKnobRight (bool pushedAgain)
{
    resetTimer ();
    if (enabled && _cur + _increment <= _high) {
        _cur += _increment;
        update (_cur);
        draw ();
        refresh ();
    }
}

bool
Slider::_onMenuButton (bool pushedAgain)
{
    switchToParent ();
}

void
Slider::setCur (double cur)
{
    _cur = cur;
    update (cur);
    draw ();
    refresh ();
}

void
Slider::disable (void)
{
    enabled = false;
    draw_disabled ();
    refresh ();
}

void
Slider::enable (void)
{
    enabled = true;
    draw ();
    refresh ();
}

void
Slider::update (double val)
{
    cerr << "Warning: default update called for slider " << name() << endl;
}

void
Slider::inFocus (void)
{
    resetTimer ();
}

void
Slider::outFocus (void)
{
    clearTimer ();
}
